;--------------------------------------
; Experimental modem driver for DUART card
; The DUART card is the device developped
; by the IDE64 Project team.
; For more information:
; http://ide64.come.to
;
; 7.6.2000 (c) Josef Soucek
;
; Configured for channel A
;--------------------------------------
;LONGSHINE ext.modem LCS-8824
;supports:
;start b, data b, parity, stop b.
;1,7,even or odd, 1
;1,7,no, 2
;1,7,mark or space, 1
;1,8,no,1
;--------------------------------------

duart    = $de00


         *= $1000
;--------------
;nast. po linkeru
         lda #$37
         sta $01
         cli
;--------------

         jsr set_indicator
         lda #3            ;3=2400
         jsr set_tr_rate
         jsr set_mode      ;1,8,no,1

         lda #%00000101    ;enable tx,rx
         sta duart+$02     ;CRA
;------------------
receive_loop = *
         lda duart+$01   ;SRA
         and #%00000001  ;RXRDY
         beq receive_cont

         lda duart+$03   ;RHRA
         jsr $ffd2            ;Chrout
receive_cont = *
         jsr $ffe4            ;Getin
         cmp #0
         beq receive_loop
         tax
         lda duart+$01
         and #%00000100
         beq receive_loop
         stx duart+$03   ;THRA
         jmp receive_loop

         rts

;--------------------------------------
; Set mode registers
;
set_mode = *
         lda #%00010000   ;Reset MR pointer
         sta duart+$02    ;CRA
         ;
         lda #%00010011   ;8bit no par
         sta duart+$00    ;MR1A
         lda #%00000111   ;1 stop bit
         sta duart+$00    ;MR2A
         rts

;--------------------------------------
; Set input/output transfer rate
;
; Input A=0 - 110    bps
;         1 - 300    bps
;         2 - 1200   bps
;         3 - 2400   bps
;         4 - 4800   bps
;         5 - 9600   bps
;         6 - 19200  bps
;         7 - 38400  bps
;         8 - 57600  bps
;         9 - 115200 bps
;--------------------------------------
set_tr_rate    = *
               asl
               tay
               lda tr_rate_tab,y
               beq set_tr_rate_x0
               ;
               lda #%10100000       ;tr X=1
               sta duart+$02        ;CRA
               lda #%10000000       ;rx X=1
               sta duart+$02        ;CRA
               jmp set_tr_rate_cont
set_tr_rate_x0 = *
               lda #%10110000       ;tr X=0
               sta duart+$02        ;CRA
               lda #%10010000       ;rx X=0
               sta duart+$02        ;CRA
set_tr_rate_cont = *
               iny
               lda tr_rate_tab,y    ;low=tx hi=rx
               sta duart+$01        ;CSRA
               lda #%00000000       ;Bit R.Set #1
               sta duart+$04        ;ACR [bit 7]
               rts
;--------------------------------------
; Bit Rate Set #1
;--------------------------------------
tr_rate_tab    .byte 0,%00010001   ;110 X,lo=tx hi=rx
               .byte 0,%01000100   ;300
               .byte 0,%01100110   ;1200
               .byte 0,%10001000   ;2400
               .byte 0,%10011001   ;4800
               .byte 0,%10111011   ;9600
               .byte 1,%11001100   ;19200
               .byte 0,%11001100   ;38400
               .byte 1,%01110111   ;57600
               .byte 1,%10001000   ;115200
;--------------------------------------
; Set indicator channel A
;
set_indicator = *
         lda #%01010000
         sta duart+$0d   ;OPCR
         rts

