;--------------------------------------
; Experimental PC mouse driver
; for DUART 68C681 card
; The DUART card is the device developped
; by the IDE64 Project team.
; For more information:
; http://ide64.come.to
;
; 9.12.1999 by Josef Soucek (c)
; (based on unknown sourcecode)
;
;--------------------------------------
; mouse  7bit,1stop,1200baud
;
; set TxD(3)=-12V  (genius mouse don't need -12V)
; set DTR(4)=+12V
; set RTS(7)=+12V  (5,24V only buttons)
;                  (OmniBook 7,6V)
;--------------------------------------
duart    = $de00         ;DUART base address


         *= $1000

         jsr set_indicator
         jsr set_tr_rate
         jsr set_mode

         lda #%00001001  ;disable tx, enable rx
         sta duart+$02   ;CRA

         lda #%00000101  ;RTS(op0)=+12, DTR(op2)=+12
         sta duart+$0e   ;SOPBC
         ;
         jmp mouse_driver
         
;--------------------------------------
; Set mode registers
;
set_mode = *
         lda #%00010000   ;Rst MR point
         sta duart+$02    ;CRA
         ;
         lda #%00010010   ;7bit no par
         sta duart+$00    ;MR1A
         lda #%00000111   ;1 stop bit
         sta duart+$00    ;MR2A
         rts

;--------------------------------------
; Set input/output transfer rate
;
set_tr_rate = *
;         lda #%10110000  ;transmit X=0
;         sta duart+$02   ;CRA

         lda #%10010000  ;receiver X=0
         sta duart+$02   ;CRA
         ;
         lda #%10000000  ;Bit R.Set #2
         sta duart+$04   ;ACR
         ;
         lda #%01100110  ;1200
         sta duart+$01   ;CSRA l=tx h=r
         rts

;--------------------------------------
; Set indicator channel A
;
set_indicator = *
         lda #%01010000
         sta duart+$0d   ;OPCR
         rts
;--------------------------------------
; MOUSE DRIVER
;--------------------------------------

mouse_driver = *
         sei
         lda #1
         sta 53248+21              ;Turn-on pointing sprite

ReadLoop ldx #$00       

receive_no = *
         lda duart+$01   ;SRA
         and #%00000001  ;RXRDY    Wait until byte appear
         beq receive_no
         
         inc $d020

         lda duart+$03   ;RHRA
         sta $0400,x               ;read 3 bytes
         inx
         cpx #$03
         bne receive_no
         
                  
         ldx $0400
         
;------------------------------------------ test X
move_x   txa 
         and #%00000011
         beq move_r
                  
;------------------------------------------ move left
         lda $0401              ; speed (R/L)
         eor #%00111111         ; for move left INVERT!!!!
         sta $0500
         
         lda $d000              ; current pointer position
         sec
         sbc $0500              ; subtrack the speed
         sta $d000
         bcs move_y
         lda #$01
         eor $d010              ; high bite correction
         sta $d010
         jmp move_y             ; continue

;------------------------------------------ move right
move_r   lda $0401              ; speed (R/L)
         clc
         adc $d000              
         sta $d000              ; current sprite pointer position
         bcc move_y
         lda #$01
         eor $d010
         sta $d010              ; high bite correction
         
;----------------------------------------- test Y
move_y   txa
         and #%00001100
         beq move_d

;----------------------------------------- move up
         lda $0402              ; speed
         eor #%00111111
         sta $0501
         lda $d001
         sec
         sbc $0501
         sta $d001              ; new sprite pointer position
         jmp ReadLoop

;------------------------------------------ move down
move_d   lda $0402             ; speed U/D
         clc
         adc $d001
         sta $d001             ; new sprite pointer position
         jmp ReadLoop


